
from xepmts_endpoints import get_endpoints

GLOBAL_WRITE_ROLES = ["admin", "expert"]
GLOBAL_READ_ROLES = ["read:all"]

def get_domain(global_write_roles=GLOBAL_WRITE_ROLES, global_read_roles=GLOBAL_READ_ROLES):
    endpoints = get_endpoints()
    for endpoint in endpoints.values():
        experiment = endpoint.get("experiment", "xenonnt")
        detector = endpoint.get("detector", "tpc")
        
        write_roles = global_write_roles + [f'write:{experiment}', 
                                             f'write:{experiment}_{detector}', 
                                             f'write:{endpoint["datasource"]["source"]}']
        read_roles = write_roles + global_read_roles + [ f'read:{experiment}',
                                             f'read:{experiment}_{detector}',
                                             f'read:{endpoint["datasource"]["source"]}']

        endpoint["allowed_read_roles"] = endpoint.get("allowed_read_roles", []) + read_roles
        endpoint["allowed_item_read_roles"] = endpoint.get("allowed_item_read_roles", []) + read_roles
        endpoint["allowed_write_roles"] = endpoint.get("allowed_write_roles", []) + write_roles
        endpoint["allowed_item_write_roles"] = endpoint.get("allowed_item_write_roles", []) + write_roles
    return endpoints
    


# -*- coding: utf-8 -*-
import os

from xepmts_server.v2.domain import get_domain
from xepmts_server.v2 import settings


def make_app(swagger=False, export_metrics=False, **kwargs):
    from eve import Eve
    
    app = Eve(settings=settings.__file__, **kwargs)
    if swagger:
        # from eve_swagger import swagger as swagger_blueprint
        from eve_swagger import get_swagger_blueprint
        
        swagger_blueprint = get_swagger_blueprint()
        app.register_blueprint(swagger_blueprint, url_prefix=f'/{app.config["API_VERSION"]}')
        app.config['SWAGGER_INFO'] = {
            'title': 'XENON PMT API',
            'version': '2.0',
            'description': 'API for the XENON PMT database',
            'termsOfService': 'https://opensource.org/ToS',
            'contact': {
                'name': 'Yossi Mosbacher',
                'url': 'https://pmts.xenonnt.org',
                "email": "joe.mosbacher@gmail.com"
            },

            'license': {
                'name': 'BSD',
                'url': 'https://github.com/nicolaiarocci/eve-swagger/blob/master/LICENSE',
            
            },
            'schemes': ['http', 'https'],

        }

    if export_metrics:
        from prometheus_flask_exporter import PrometheusMetrics
        PrometheusMetrics(app)

    return app


def make_local_app(**kwargs):
    import eve
    app = eve.Eve(settings=settings.__file__, **kwargs)
    return app
    
def list_roles():
    domain = get_domain()
    roles = set()
    for resource in domain.values():
        roles.update(resource["allowed_read_roles"])
        roles.update(resource["allowed_item_read_roles"])
        roles.update(resource["allowed_write_roles"])
        roles.update(resource["allowed_item_write_roles"])
    roles = list(roles)
    roles.sort(key=lambda x: x.split(":")[-1])
    return roles


from xepmts_server.v1.app import make_local_app
import os
import pkg_resources

# APPS = {
#     entry_point.name: entry_point.load()
#     for entry_point
#     in pkg_resources.iter_entry_points('xepmts.apps')
# }

# APPS["db"] =  make_local_app

def make_client(app_names=None):
    import eve_panel

    # if app_names is None:
    #     app_names = list(APPS)
    # apps = {name: APPS[name]() for name in app_names}
    app = make_local_app()
    client = eve_panel.EveClient.from_app(app, name="xepmts", sort_by_url=True)
    client._http_client.auth.set_auth_by_name("Bearer")
    client.db = client
    # for name in apps:
    #     getattr(client, name)._http_client.auth.set_auth_by_name("Bearer")
    return client

def default_client():
    return make_client()


from . import v1
from . import v2
from .server import run_simple, create_app

__author__ = """Yossi Mosbacher"""
__email__ = 'joe.mosbacher@gmail.com'
__version__ = '0.1.3'

VERSIONS = {
    "v1": v1,
    "v2": v2,
}

DEFAULT_VERSION = "v2"

def get_server(version, **kwargs):
    app = VERSIONS[version].app.make_app(**kwargs)
    return app

def default_server():
    return get_server(DEFAULT_VERSION)




# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['tests',
 'xepmts_server',
 'xepmts_server.sc',
 'xepmts_server.v1',
 'xepmts_server.v2']

package_data = \
{'': ['*']}

install_requires = \
['Eve-Swagger>=0.1.3,<0.2.0',
 'Eve>=1.1.5,<2.0.0',
 'eve-jwt>=0.1.9,<0.2.0',
 'flask_swagger_ui>=3.36.0,<4.0.0',
 'prometheus_flask_exporter>=0.18.1,<0.19.0',
 'toml>=0.10.2,<0.11.0',
 'xeauth==0.1.2',
 'xepmts-endpoints>=0.1.0,<0.2.0']

entry_points = \
{'console_scripts': ['xepmts_server = xepmts_server.cli:main']}

setup_kwargs = {
    'name': 'xepmts-server',
    'version': '0.1.3',
    'description': 'Top-level package for xepmts-server.',
    'long_description': '=============\nxepmts-server\n=============\n\n\n.. image:: https://img.shields.io/pypi/v/xepmts_server.svg\n        :target: https://pypi.python.org/pypi/xepmts_server\n\n.. image:: https://img.shields.io/travis/jmosbacher/xepmts_server.svg\n        :target: https://travis-ci.com/jmosbacher/xepmts_server\n\n.. image:: https://readthedocs.org/projects/xepmts-server/badge/?version=latest\n        :target: https://xepmts-server.readthedocs.io/en/latest/?badge=latest\n        :alt: Documentation Status\n\n\n\n\nServer for the xepmts api\n\n\n* Free software: MIT\n* Documentation: https://xepmts-server.readthedocs.io.\n\n\nFeatures\n--------\n\n* TODO\n\nCredits\n-------\n\nThis package was created with Cookiecutter_ and the `briggySmalls/cookiecutter-pypackage`_ project template.\n\n.. _Cookiecutter: https://github.com/audreyr/cookiecutter\n.. _`briggySmalls/cookiecutter-pypackage`: https://github.com/briggySmalls/cookiecutter-pypackage\n',
    'author': 'Yossi Mosbacher',
    'author_email': 'joe.mosbacher@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/jmosbacher/xepmts_server',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.7.1',
}


setup(**setup_kwargs)




GLOBAL_WRITE_ROLES = ["admin", "expert"]
GLOBAL_READ_ROLES = ["read:all"]

def add_roles(resources,
              global_write_roles=GLOBAL_WRITE_ROLES,
              global_read_roles=GLOBAL_READ_ROLES):

    new_resources = {}
    for name, resource in resources.items():
        experiment_name = resource["schema"]["experiment"]["default"]
        detector = resource["schema"]["detector"]["default"]

        write_roles = global_write_roles + [f'write:{experiment_name}', 
                                                f'write:{experiment_name}_{detector}', 
                                                f'write:{resource["datasource"]["source"]}']
        read_roles = write_roles + global_read_roles + [ f'read:{experiment_name}',
                                        f'read:{experiment_name}_{detector}',
                                        f'read:{resource["datasource"]["source"]}']
        new_resource = dict(resource)
        new_resource["allowed_read_roles"] = resource.get("allowed_read_roles", []) + read_roles
        new_resource["allowed_item_read_roles"] = resource.get("allowed_item_read_roles", []) + read_roles
        new_resource["allowed_write_roles"] = resource.get("allowed_write_roles", []) + write_roles
        new_resource["allowed_item_write_roles"] = resource.get("allowed_item_write_roles", []) + write_roles
        new_resources[name] = new_resource
    return new_resources


import os

from copy import deepcopy
from xepmts_server.v2.secrets import MONGO_PASSWORD
from xepmts_server.v2.domain import get_domain

DOMAIN = get_domain()

URL_PREFIX = os.getenv("XEPMTS_URL_PREFIX", "")
API_VERSION = "v2"
RESOURCE_METHODS = ["GET", "POST",]
ITEM_METHODS = ["GET", "PUT", "PATCH", "DELETE"]
ALLOWED_READ_ROLES = ["admin", "superuser", "expert", "user", "read", "write"]
ALLOWED_WRITE_ROLES = ["admin", "superuser", "expert", "write"]
EMBEDDING = True
MEDIA_PATH = "files"
PAGINATION_LIMIT = 10000
SCHEMA_ENDPOINT = "schema"
IF_MATCH = True
ENFORCE_IF_MATCH = False
HATEOAS = True
VERSIONS = "_versions"
NORMALIZE_ON_PATCH = False

# ----------------- Mongo config ------------------------------------------ #
MONGO_HOST = os.getenv("XEPMTS_MONGO_HOST", "localhost")
MONGO_PORT = int(os.getenv("XEPMTS_MONGO_PORT", 27017))
MONGO_DBNAME = os.getenv("XEPMTS_MONGO_DB", "pmts")
MONGO_USERNAME = os.getenv("XEPMTS_MONGO_USER", "")
MONGO_AUTH_SOURCE = os.getenv("XEPMTS_MONGO_AUTH_SOURCE", MONGO_DBNAME)

MONGO1T_HOST = MONGO_HOST
MONGO1T_PORT = MONGO_PORT
MONGO1T_DBNAME = MONGO_DBNAME + "1t"
MONGO1T_AUTH_SOURCE = MONGO_AUTH_SOURCE
MONGO1T_PASSWORD = MONGO_PASSWORD
if os.getenv("XEPMTS_MONGO_REPLICA_SET", ""):
    MONGO_REPLICA_SET = os.getenv("XEPMTS_MONGO_REPLICA_SET", "")
    MONGO1T_REPLICA_SET = MONGO_REPLICA_SET

MONGO1T_USERNAME = MONGO_USERNAME

if os.getenv("XEPMTS_MONGO_URI", ""):
    MONGO_URI = os.getenv("XEPMTS_MONGO_URI", "")
    MONGO1T_URI = MONGO_URI
# -------------------------------------------------------------------------- #

SERVERS = [
    "https://api-dot-xenon-pmts.uc.r.appspot.com",
    "https://api."+os.getenv('XEPMTS_DOMAIN','pmts.xenonnt.org'),        
    ]

X_DOMAINS = ['http://localhost:8000',
            'http://127.0.0.1:8000',
            'http://127.0.0.1:5000',
            'http://editor.swagger.io',
            "https://"+os.getenv('XEPMTS_DOMAIN','pmts.xenonnt.org'),
            "https://api."+os.getenv('XEPMTS_DOMAIN','pmts.xenonnt.org'),
            "https://website."+os.getenv('XEPMTS_DOMAIN','pmts.xenonnt.org'),
            "https://panels."+os.getenv('XEPMTS_DOMAIN','pmts.xenonnt.org'),
            "https://catalog."+os.getenv('XEPMTS_DOMAIN','pmts.xenonnt.org'),
             ]

X_HEADERS = ['Content-Type', 'If-Match', 'Authorization', 'X-HTTP-Method-Override']  # Needed for the "Try it out" buttons

JWT_AUDIENCES = os.getenv("XEPMTS_JWT_AUDIENCES", "https://api.pmts.xenonnt.org").split(",")
JWT_KEY_URL = os.getenv("XEPMTS_JWT_KEY_URL", "https://auth-dot-xenon-pmts.uc.r.appspot.com/.well-known/jwks.json")
JWT_SCOPE_CLAIM = os.getenv("XEPMTS_JWT_SCOPE_CLAIM","scope")
JWT_ROLES_CLAIM = os.getenv("XEPMTS_JWT_ROLES_CLAIM", "roles")
JWT_TTL = 3600
